// C/C++ headers

#include <stdio.h>
#include <stdlib.h>

// QALE headers

#include "BinaryNode.h"
#include "cpp_assert.h"
#include "Visitor.h"

/*****************************************************************************
 * Definition of class: BinaryNode
 *****************************************************************************/

BinaryNode::BinaryNode(
   const Node *left,
   Operator theOperator,
   const Node *right
)
   :
   mLeft(left),
   mOperator(theOperator),
   mRight(right)
{
   // Nothing to do
}

BinaryNode::~BinaryNode()
{
   // Must cast to non-const before deletion

   delete (Node *)mLeft;
   delete (Node *)mRight;
}

double BinaryNode::evaluate() const
{
   double result = 0.0;

   switch(mOperator)
   {
      case _PLUS_  :
      {
         result = mLeft->evaluate() + mRight->evaluate();
      }
      break;

      case _MINUS_ :
      {
         result = mLeft->evaluate() - mRight->evaluate();
      }
      break;

      case _TIMES_ :
      {
         result = mLeft->evaluate() * mRight->evaluate();
      }
      break;

      case _DIVIDE_:
      {
         result = mLeft->evaluate() / mRight->evaluate();
      }
      break;

      default:
      {
         // Shouldn't be here

         CPP_ASSERT(0);
      }
      break;
   }

   return(result);
}

ostream &BinaryNode::print(ostream &out) const
{
   return(
      out << "(" << *mLeft << ") " <<
      operatorAsString() << " (" << *mRight << ")"
   );
}

Node *BinaryNode::copy() const
{
   return(
      new BinaryNode(
         mLeft->copy(),
         mOperator,
         mRight->copy()
      )
   );
}

void BinaryNode::accept(Visitor &v) const
{
   v.visited((BinaryNode *)this);

   mLeft->accept(v);
   mRight->accept(v);
}

const char *BinaryNode::operatorAsString() const
{
   char *asString = NULL;

   switch(mOperator)
   {
      case _PLUS_  : asString = "+"; break;
      case _MINUS_ : asString = "-"; break;
      case _TIMES_ : asString = "*"; break;
      case _DIVIDE_: asString = "/"; break;

      default:
      {
         // We shouldn't ever be here

         CPP_ASSERT(0);
      }
   }

   return(asString);
}
