// C/C++ headers

#include <string.h>
#include <stdio.h>

// QALE headers

#include "DefineNode.h"
#include "Visitor.h"

/*****************************************************************************
 * Definition of class: DefineNode
 *****************************************************************************/

DefineNode::DefineNode(const char *defineName)
   :
   mQualifiedName(NULL),
   mDefineName(NULL),
   mResolved(0)
{
   Node::split(defineName, &mQualifiedName, &mDefineName);
}

DefineNode::~DefineNode()
{
   delete [] mQualifiedName;
   delete [] mDefineName;
   delete mResolved;
}

double DefineNode::evaluate() const
{
   return(mResolved->evaluate());
}

ostream &DefineNode::print(ostream &out) const
{
   out << "$";

   if(*mQualifiedName)
      out << mQualifiedName << ".";

   return(out << mDefineName);
}

Node *DefineNode::copy() const
{
   char *defineName = Node::join(mQualifiedName, mDefineName);
   Node *n          = new DefineNode(defineName);

   delete [] defineName;

   return(n);
}

void DefineNode::accept(Visitor &v) const
{
   v.visited((DefineNode *)this);

   if(mResolved)
      mResolved->accept(v);
}

char *DefineNode::getQualifiedName() const
{
   return(mQualifiedName);
}

char *DefineNode::getDefineName() const
{
   return(mDefineName);
}

void DefineNode::setNode(Node *n)
{
   mResolved = n;
}
