// C/C++ headers

#include <string.h>
#include <stdio.h>

// QALE headers

#include "PropertyNode.h"
#include "Visitor.h"
#include "Component.h"

/*****************************************************************************
 * Definition of class: PropertyNode
 *****************************************************************************/

PropertyNode::PropertyNode(const char *propertyName)
   :
   mQualifiedName(NULL),
   mPropertyName(NULL)
{
   Node::split(propertyName, &mQualifiedName, &mPropertyName);
}

PropertyNode::~PropertyNode()
{
   delete [] mPropertyName;
   delete [] mQualifiedName;
}

double PropertyNode::evaluate() const
{
   return(mComponent->getProperty(mPropertyName));
}

ostream &PropertyNode::print(ostream &out) const
{
   if(*mQualifiedName)
      out << mQualifiedName << ".";

   return(out << mPropertyName << "()");
}

Node *PropertyNode::copy() const
{
   char *propertyName = Node::join(mQualifiedName, mPropertyName);
   Node *n          = new PropertyNode(propertyName);

   delete [] propertyName;

   return(n);
}

void PropertyNode::accept(Visitor &v) const
{
   v.visited((PropertyNode *)this);
}

char *PropertyNode::getQualifiedName() const
{
   return(mQualifiedName);
}

char *PropertyNode::getPropertyName() const
{
   return(mPropertyName);
}

void PropertyNode::setComponent(Component *c)
{
   mComponent = c;
}
