#ifndef _VISITOR_H_
#   define _VISITOR_H_

#include <string>

using std::string;

class Node;
class AttributeNode;
class BinaryNode;
class ConstantNode;
class DefineNode;
class PropertyNode;
class MinMaxNode;
class UnaryNode;
class Widget;

class Visitor
{
   public:
      typedef enum
      {
         UNKNOWN, SIZE_WIDTH, SIZE_HEIGHT, X_LOCATION, Y_LOCATION

      } Field;

      Visitor();
      virtual ~Visitor();

      virtual void visited(AttributeNode *) = 0;
      virtual void visited(BinaryNode *)    = 0;
      virtual void visited(ConstantNode *)  = 0;
      virtual void visited(DefineNode *)    = 0;
      virtual void visited(PropertyNode *)  = 0;
      virtual void visited(MinMaxNode *)    = 0;
      virtual void visited(UnaryNode *)     = 0;

      void setCurrentWidget(Widget *);
      void setField(Field field);

   protected:
      Widget *getCurrentWidget();
      Widget *findWidget(const char *qualifiedName);
      string  getErrorMessage(const char *text, const Node *n);
      Field getField();

   private:
      Visitor(const Visitor &);
      Visitor &operator=(const Visitor &);
      const char *fieldAsString() const;

      Widget *mCurrentWidget;
      Field mField;
};

#endif
